@@ -1,13 +1,29 @@
 """Control tasks execution order"""
 import fnmatch
 from collections import deque
+from collections import OrderedDict
+import re
 import six
 
 from .exceptions import InvalidTask, InvalidCommand, InvalidDodoFile
+from .cmdparse import TaskParse, CmdOption
 from .task import Task, DelayedLoaded
 from .loader import generate_tasks
 
 
+class RegexGroup(object):
+    '''Helper to keep track of all delayed-tasks which regexp target
+    matches the target specified from command line.
+    '''
+    def __init__(self, target, tasks):
+        # target name specified in command line
+        self.target = target
+        # set of delayed-tasks names (string)
+        self.tasks = tasks
+        # keep track if the target was already found
+        self.found = False
+
+
 class TaskControl(object):
     """Manages tasks inter-relationship
 
@@ -26,9 +42,10 @@ class TaskControl(object):
                           Value: task_name
     """
 
-    def __init__(self, task_list):
-        self.tasks = {}
+    def __init__(self, task_list, auto_delayed_regex=False):
+        self.tasks = OrderedDict()
         self.targets = {}
+        self.auto_delayed_regex = auto_delayed_regex
 
         # name of task in order to be executed
         # this the order as in the dodo file. the real execution
@@ -140,7 +157,8 @@ class TaskControl(object):
                 # parse task_selection
                 the_task = self.tasks[f_name]
                 # remaining items are other tasks not positional options
-                the_task.options, seq = the_task.taskcmd.parse(seq)
+                taskcmd = TaskParse([CmdOption(opt) for opt in the_task.params])
+                the_task.options, seq = taskcmd.parse(seq)
                 # if task takes positional parameters set all as pos_arg_val
                 if the_task.pos_arg is not None:
                     the_task.pos_arg_val = seq
@@ -174,14 +192,53 @@ class TaskControl(object):
             # by task name
             if filter_ in self.tasks:
                 selected_task.append(filter_)
+                continue
+
             # by target
-            elif filter_ in self.targets:
+            if filter_ in self.targets:
                 selected_task.append(self.targets[filter_])
-            else:
-                msg = ('cmd `run` invalid parameter: "%s".' +
-                       'Must be a task, or a target.\n' +
-                       'Type "doit list" to see available tasks')
-                raise InvalidCommand(msg % filter_)
+                continue
+
+            # if can not find name check if it is a sub-task of a delayed
+            basename = filter_.split(':', 1)[0]
+            if basename in self.tasks:
+                loader = self.tasks[basename].loader
+                if not loader:
+                    raise InvalidCommand(not_found=filter_)
+                loader.basename = basename
+                self.tasks[filter_] = Task(filter_, None, loader=loader)
+                selected_task.append(filter_)
+                continue
+
+            # check if target matches any regex
+            delayed_matched = []  # list of Task
+            for task in list(self.tasks.values()):
+                if not task.loader:
+                    continue
+                if task.name.startswith('_regex_target'):
+                    continue
+                if task.loader.target_regex:
+                    if re.match(task.loader.target_regex, filter_):
+                        delayed_matched.append(task)
+                elif self.auto_delayed_regex:
+                    delayed_matched.append(task)
+            delayed_matched_names = [t.name for t in delayed_matched]
+            regex_group = RegexGroup(filter_, set(delayed_matched_names))
+
+            # create extra tasks to load delayed tasks matched by regex
+            for task in delayed_matched:
+                loader = task.loader
+                loader.basename = task.name
+                name = '{}_{}:{}'.format('_regex_target', filter_, task.name)
+                loader.regex_groups[name] = regex_group
+                self.tasks[name] = Task(name, None,
+                                        loader=loader,
+                                        file_dep=[filter_])
+                selected_task.append(name)
+
+            if not delayed_matched:
+                # not found
+                raise InvalidCommand(not_found=filter_)
         return selected_task
 
 
@@ -213,7 +270,7 @@ class ExecNode(object):
     """Each task will have an instace of this
     This used to keep track of waiting events and the generator for dep nodes
 
-    @ivar run_status (str): contains the result of Dependency.get_status
+    @ivar run_status (str): contains the result of Dependency.get_status().status
             modified by runner, value can be:
            - None: not processed yet
            - run: task is selected to be executed (it might be running or
@@ -256,7 +313,6 @@ class ExecNode(object):
 
     def reset_task(self, task, generator):
         """reset task & generator after task is created by its own `loader`"""
-        task.loader = DelayedLoaded
         self.task = task
         self.task_dep = task.task_dep[:]
         self.calc_dep = task.calc_dep.copy()
@@ -371,6 +427,13 @@ class TaskDispatcher(object):
         """
         this_task = node.task
 
+        # skip this task if task belongs to a regex_group that already
+        # executed the task used to build the given target
+        if this_task.loader:
+            regex_group = this_task.loader.regex_groups.get(this_task.name, None)
+            if regex_group and regex_group.found:
+                return
+
         # add calc_dep & task_dep until all processed
         # calc_dep may add more deps so need to loop until nothing left
         while True:
@@ -396,14 +459,39 @@ class TaskDispatcher(object):
             else:
                 break
 
+        # generate tasks from a DelayedLoader
         if this_task.loader:
             ref = this_task.loader.creator
-            new_tasks = generate_tasks(this_task.name, ref(), ref.__doc__)
-            TaskControl.set_implicit_deps(self.targets, new_tasks)
-            for nt in new_tasks:
-                if not nt.loader:
-                    nt.loader = DelayedLoaded
-                self.tasks[nt.name] = nt
+            to_load = this_task.loader.basename or this_task.name
+            this_loader = self.tasks[to_load].loader
+            if this_loader and not this_loader.created:
+                new_tasks = generate_tasks(to_load, ref(), ref.__doc__)
+                TaskControl.set_implicit_deps(self.targets, new_tasks)
+                for nt in new_tasks:
+                    if not nt.loader:
+                        nt.loader = DelayedLoaded
+                    self.tasks[nt.name] = nt
+            # check itself for implicit dep (used by regex_target)
+            TaskControl.add_implicit_task_dep(
+                self.targets, this_task, this_task.file_dep)
+
+            # remove file_dep since generated tasks are not required
+            # to really create the target (support multiple matches)
+            if regex_group:
+                this_task.file_dep = {}
+                if regex_group.target in self.targets:
+                    regex_group.found = True
+                else:
+                    regex_group.tasks.remove(this_task.loader.basename)
+                    if len(regex_group.tasks) == 0:
+                        # In case no task is left, we cannot find a task
+                        # generating this target. Print an error message!
+                        raise InvalidCommand(not_found=regex_group.target)
+
+            # mark this loader to not be executed again
+            this_task.loader.created = True
+            this_task.loader = DelayedLoaded
+
             # this task was placeholder to execute the loader
             # now it needs to be re-processed with the real task
             yield "reset generator"
