@@ -4,6 +4,7 @@ import os
 import hashlib
 import subprocess
 import inspect
+from collections import defaultdict
 import six
 if six.PY3: # pragma: no cover
     from dbm import dumb
@@ -25,13 +26,11 @@ import json
 import sqlite3
 
 
-USE_FILE_TIMESTAMP = True
-
-
 class DatabaseException(Exception):
     """Exception class for whatever backend exception"""
     pass
 
+
 def get_md5(input_data):
     """return md5 from string or unicode"""
     if isinstance(input_data, six.text_type):
@@ -40,6 +39,7 @@ def get_md5(input_data):
         byte_data = input_data
     return hashlib.md5(byte_data).hexdigest()
 
+
 def get_file_md5(path):
     """Calculate the md5 sum from file content.
 
@@ -57,28 +57,6 @@ def get_file_md5(path):
     return md5.hexdigest()
 
 
-
-def check_modified(file_path, file_stat, state):
-    """check if file in file_path is modified from previous "state"
-    @param file_path (string): file path
-    @param file_stat: the value returned from os.stat(file_path)
-    @param state (tuple), timestamp, size, md5
-    @returns (bool):
-    """
-    if state is None:
-        return True
-
-    timestamp, size, file_md5 = state
-    # 1 - if timestamp is not modified file is the same
-    if USE_FILE_TIMESTAMP and file_stat.st_mtime == timestamp:
-        return False
-    # 2 - if size is different file is modified
-    if file_stat.st_size != size:
-        return True
-    # 3 - check md5
-    return file_md5 != get_file_md5(file_path)
-
-
 class JsonDB(object):
     """Backend using a single text file with JSON content"""
 
@@ -282,6 +260,8 @@ class SqliteDB(object):
     def __init__(self, name):
         self.name = name
         self._conn = self._sqlite3(self.name)
+        self._cache = {}
+        self._dirty = set()
 
     @staticmethod
     def _sqlite3(name):
@@ -301,7 +281,7 @@ class SqliteDB(object):
         conn = sqlite3.connect(
             name,
             detect_types=sqlite3.PARSE_DECLTYPES|sqlite3.PARSE_COLNAMES,
-            isolation_level=None)
+            isolation_level='DEFERRED')
         conn.row_factory = dict_factory
         sqlscript = """
             create table if not exists doit (
@@ -326,8 +306,11 @@ class SqliteDB(object):
 
         @return: (string) or (None) if entry not found
         """
-        data = self._get_task_data(task_id)
-        return data.get(dependency, None)
+        if task_id in self._cache:
+            return self._cache[task_id].get(dependency, None)
+        else:
+            data = self._cache[task_id] = self._get_task_data(task_id)
+            return data.get(dependency, None)
 
     def _get_task_data(self, task_id):
         data = self._conn.execute('select task_data from doit where task_id=?',
@@ -336,12 +319,15 @@ class SqliteDB(object):
 
     def set(self, task_id, dependency, value):
         """Store value in the DB."""
-        task_data = self._get_task_data(task_id)
-        task_data[dependency] = value
-        self._conn.execute('insert or replace into doit values (?,?)',
-                           (task_id, task_data))
+        if task_id not in self._cache:
+            self._cache[task_id] = {}
+        self._cache[task_id][dependency] = value
+        self._dirty.add(task_id)
+
 
     def in_(self, task_id):
+        if task_id in self._cache:
+            return True
         if self._conn.execute('select task_id from doit where task_id=?',
                               (task_id,)).fetchone():
             return True
@@ -349,21 +335,150 @@ class SqliteDB(object):
 
     def dump(self):
         """save/close sqlite3 DB file"""
+        for task_id in self._dirty:
+            self._conn.execute('insert or replace into doit values (?,?)',
+                               (task_id, json.dumps(self._cache[task_id])))
         self._conn.commit()
         self._conn.close()
+        self._dirty = set()
 
     def remove(self, task_id):
         """remove saved dependecies from DB for taskId"""
+        if task_id in self._cache:
+            del self._cache[task_id]
+        if task_id in self._dirty:
+            self._dirty.remove(task_id)
         self._conn.execute('delete from doit where task_id=?', (task_id,))
 
     def remove_all(self):
         """remove saved dependecies from DB for all task"""
         self._conn.execute('delete from doit')
+        self._cache = {}
+        self._dirty = set()
+
+
+class FileChangedChecker(object):
+    """Base checker for dependencies, must be inherited."""
+
+    def check_modified(self, file_path, file_stat, state):
+        """Check if file in file_path is modified from previous "state".
+
+        @param file_path (string): file path
+        @param file_stat: result of os.stat() of file_path
+        @param state: state that was previously saved with ``get_state()``
+        @returns (bool): True if dep is modified
+
+        """
+        raise NotImplementedError()
+
+    def get_state(self, dep, current_state):
+        """Compute the state of a task after it has been successfuly executed.
+
+        @param dep (str): path of the dependency file.
+        @param current_state (tuple): the current state, saved from a previous
+            execution of the task (None if the task was never run).
+        @returns: the new state. Return None if the state is unchanged.
+
+        The parameter `current_state` is passed to allow speed optimization,
+        see MD5Checker.get_state().
+        """
+        raise NotImplementedError()
+
+
+class MD5Checker(FileChangedChecker):
+    """MD5 checker, uses the md5sum.
+
+    This is the default checker used by doit.
+
+    As an optimization the check uses (timestamp, file-size, md5).
+    If the timestamp is the same it considers that the file has the same
+    content. If file size is different its content certainly is modified.
+    Finally the md5 is used for a different timestamp with the same size.
+    """
+
+    def check_modified(self, file_path, file_stat, state):
+        """Check if file in file_path is modified from previous "state".
+        """
+        timestamp, size, file_md5 = state
+
+        # 1 - if timestamp is not modified file is the same
+        if file_stat.st_mtime == timestamp:
+            return False
+
+        # 2 - if size is different file is modified
+        if file_stat.st_size != size:
+            return True
+
+        # 3 - check md5
+        return file_md5 != get_file_md5(file_path)
+
+
+    def get_state(self, dep, current_state):
+        timestamp = os.path.getmtime(dep)
+        # time optimization. if dep is already saved with current
+        # timestamp skip calculating md5
+        if current_state and current_state[0] == timestamp:
+            return
+        size = os.path.getsize(dep)
+        md5 = get_file_md5(dep)
+        return timestamp, size, md5
+
+
+class TimestampChecker(FileChangedChecker):
+    """Checker that use only the timestamp."""
+
+    def check_modified(self, file_path, file_stat, state):
+        return file_stat.st_mtime != state
+
+    def get_state(self, dep, current_state):
+        """@returns float: mtime for file `dep`"""
+        return os.path.getmtime(dep)
+
+
+# name of checkers class available
+CHECKERS = {'md5': MD5Checker,
+            'timestamp': TimestampChecker}
+
+
+class DependencyStatus(object):
+    """Result object for Dependency.get_status.
+
+    @ivar status: (str) one of "run", "up-to-date" or "error"
+    """
+
+    def __init__(self, get_log):
+        self.get_log = get_log
+        self.status = 'up-to-date'
+        # save reason task is not up-to-date
+        self.reasons = defaultdict(list)
+        self.error_reason = None
+
+    def add_reason(self, reason, arg, status='run'):
+        """sets state and append reason for not being up-to-date
+        :return boolean: processing should be interrupted
+        """
+        self.status = status
+        if self.get_log:
+            self.reasons[reason].append(arg)
+        return not self.get_log
+
+    def set_reason(self, reason, arg):
+        """sets state and reason for not being up-to-date
+        :return boolean: processing should be interrupted
+        """
+        self.status = 'run'
+        if self.get_log:
+            self.reasons[reason] = arg
+        return not self.get_log
+
+    def get_error_message(self):
+        '''return str with error message'''
+        return self.error_reason
 
 
 
-class DependencyBase(object):
-    """Manage tasks dependencies (abstract class)
+class Dependency(object):
+    """Manage tasks dependencies
 
     Each dependency is saved in "db". the "db" can have json or dbm
     format where there is a dictionary for every task. each task has a
@@ -376,10 +491,11 @@ class DependencyBase(object):
     @ivar name: (string) filepath of the DB file
     @ivar _closed: (bool) DB was flushed to file
     """
-
-    def __init__(self, backend):
+    def __init__(self, db_class, backend_name, checker_cls=MD5Checker):
         self._closed = False
-        self.backend = backend
+        self.checker = checker_cls()
+        self.db_class = db_class
+        self.backend = db_class(backend_name)
         self._set = self.backend.set
         self._get = self.backend.get
         self.remove = self.backend.remove
@@ -396,34 +512,33 @@ class DependencyBase(object):
 
     ####### task specific
 
-    def save_success(self, task):
-        """save info after a task is successfuly executed"""
+    def save_success(self, task, result_hash=None):
+        """save info after a task is successfuly executed
+
+        :param result_hash: (str) explicitly set result_hash
+        """
         # save task values
         self._set(task.name, "_values_:", task.values)
 
         # save task result md5
-        if task.result:
+        if result_hash is not None:
+            self._set(task.name, "result:", result_hash)
+        elif task.result:
             if isinstance(task.result, dict):
                 self._set(task.name, "result:", task.result)
             else:
                 self._set(task.name, "result:", get_md5(task.result))
 
         # file-dep
+        self._set(task.name, 'checker:', self.checker.__class__.__name__)
         for dep in task.file_dep:
-            timestamp = os.path.getmtime(dep)
-            # time optimization. if dep is already saved with current timestamp
-            # skip calculating md5
-            current = self._get(task.name, dep)
-            if current and current[0] == timestamp:
-                continue
-            size = os.path.getsize(dep)
-            self._set(task.name, dep, (timestamp, size, get_file_md5(dep)))
+            state = self.checker.get_state(dep, self._get(task.name, dep))
+            if state is not None:
+                self._set(task.name, dep, state)
 
         # save list of file_deps
         self._set(task.name, 'deps:', tuple(task.file_dep))
 
-
-
     def get_values(self, task_name):
         """get all saved values from a task
         @return dict
@@ -445,6 +560,12 @@ class DependencyBase(object):
             raise Exception(msg % (task_id, key_name))
         return values[key_name]
 
+    def get_result(self, task_name):
+        """get the result saved from a task
+        @return dict or md5sum
+        """
+        return self._get(task_name, 'result:')
+
     def remove_success(self, task):
         """remove saved info from task"""
         self.remove(task.name)
@@ -457,23 +578,29 @@ class DependencyBase(object):
         """check if task is marked to be ignored"""
         return self._get(task.name, "ignore:")
 
-    # TODO add option to log this
-    def get_status(self, task, tasks_dict):
+    def get_status(self, task, tasks_dict, get_log=False):
         """Check if task is up to date. set task.dep_changed
 
+        If the checker class changed since the previous run, the task is
+        deleted, to be sure that its state is not re-used.
+
         @param task: (Task)
         @param tasks_dict: (dict: Task) passed to objects used on uptodate
-        @return: (str) one of up-to-date, run
+        @param get_log: (bool) if True, adds all reasons to the return
+                               object why this file will be rebuild.
+        @return: (DependencyStatus) a status object with possible status
+                                    values up-to-date, run or error
 
         task.dep_changed (list-strings): file-dependencies that are not
         up-to-date if task not up-to-date because of a target, returned value
         will contain all file-dependencies reagrdless they are up-to-date
         or not.
         """
+        result = DependencyStatus(get_log)
         task.dep_changed = []
 
         # check uptodate bool/callables
-        checked_uptodate = False
+        uptodate_result_list = []
         for utd, utd_args, utd_kwargs in task.uptodate:
             # if parameter is a callable
             if hasattr(utd, '__call__'):
@@ -512,64 +639,72 @@ class DependencyBase(object):
             # just ignored
             if uptodate_result is None:
                 continue
-            if uptodate_result:
-                checked_uptodate = True
-            else:
-                return 'run'
+            uptodate_result_list.append(uptodate_result)
+            if not uptodate_result:
+                result.add_reason('uptodate_false', (utd, utd_args, utd_kwargs))
+
+        # any uptodate check is false
+        if not get_log and result.status == 'run':
+            return result
 
         # no dependencies means it is never up to date.
-        if not (task.file_dep or checked_uptodate):
-            return 'run'
+        if not (task.file_dep or uptodate_result_list):
+            if result.set_reason('has_no_dependencies', True):
+                return result
+
 
         # if target file is not there, task is not up to date
         for targ in task.targets:
             if not os.path.exists(targ):
                 task.dep_changed = list(task.file_dep)
-                return 'run'
+                if result.add_reason('missing_target', targ):
+                    return result
+
+        # check for modified file_dep checker
+        previous = self._get(task.name, 'checker:')
+        checker_name = self.checker.__class__.__name__
+        if previous and previous != checker_name:
+            task.dep_changed = list(task.file_dep)
+            # remove all saved values otherwise they might be re-used by
+            # some optmization on MD5Checker.get_state()
+            self.remove(task.name)
+            if result.set_reason('checker_changed', (previous, checker_name)):
+                return result
 
         # check for modified file_dep
-        changed = [] # list of file_dep that changed
         previous = self._get(task.name, 'deps:')
-        if previous and set(previous) != task.file_dep:
-            status = 'run'
-        else:
-            status = 'up-to-date' # initial assumption
-        for dep in tuple(task.file_dep):
+        previous_set = set(previous) if previous else None
+        if previous_set and previous_set != task.file_dep:
+            if get_log:
+                added_files = sorted(list(task.file_dep - previous_set))
+                removed_files = sorted(list(previous_set - task.file_dep))
+                result.set_reason('added_file_dep', added_files)
+                result.set_reason('removed_file_dep', removed_files)
+            result.status = 'run'
+
+        # list of file_dep that changed
+        check_modified = self.checker.check_modified
+        changed = []
+        for dep in task.file_dep:
+            state = self._get(task.name, dep)
             try:
                 file_stat = os.stat(dep)
-            except os.error:
-                raise Exception("Dependent file '%s' does not exist." % dep)
-            if check_modified(dep, file_stat, self._get(task.name, dep)):
-                changed.append(dep)
-                status = 'run'
+            except OSError:
+                error_msg = "Dependent file '{}' does not exist.".format(dep)
+                result.error_reason = error_msg.format(dep)
+                if result.add_reason('missing_file_dep', dep, 'error'):
+                    return result
+            else:
+                if state is None or check_modified(dep, file_stat, state):
+                    changed.append(dep)
+        task.dep_changed = changed
 
-        task.dep_changed = changed #FIXME create a separate function for this
-        return status
+        if len(changed) > 0:
+            result.set_reason('changed_file_dep', changed)
+
+        return result
 
 
-####################
-
-class JsonDependency(DependencyBase):
-    """Task dependency manager with JSON backend"""
-    def __init__(self, name):
-        DependencyBase.__init__(self, JsonDB(name))
-
-class DbmDependency(DependencyBase):
-    """Task dependency manager with DBM backend"""
-    def __init__(self, name):
-        DependencyBase.__init__(self, DbmDB(name))
-
-class SqliteDependency(DependencyBase):
-    """Task dependency manager with sqlite backend"""
-    def __init__(self, name):
-        DependencyBase.__init__(self, SqliteDB(name))
-
-# map string used in cmdline option to class
-backend_map = {
-    'json': JsonDependency,
-    'dbm': DbmDependency,
-    'sqlite3': SqliteDependency,
-}
 
 #############
 
@@ -584,7 +719,3 @@ class UptodateCalculator(object):
         """@param"""
         self.get_val = dep_manager._get
         self.tasks_dict = tasks_dict
-
-
-# defaut dependency backend implementation
-Dependency = DbmDependency
